@@ -145,9 +145,9 @@ class Command(metaclass(ABCMeta)):
 
         For example::
 
-            >>> Command.set_default_command('server')
-            >>> Server.set_default_command()
-            >>> Command.set_default_command(Server)
+            >>> Command.set_default_command('server')  # doctest: +SKIP
+            >>> Server.set_default_command()           # doctest: +SKIP
+            >>> Command.set_default_command(Server)    # doctest: +SKIP
 
         '''
         if cls is Command:
