@@ -17,7 +17,7 @@
 
 Functions inside this module must not have side-effects on the
 file-system. This module re-exports (without change) several functions from the
-:py:mod:`os.path` standard module.
+:mod:`os.path` standard module.
 
 '''
 
@@ -60,7 +60,6 @@ Example::
     '.../tmp/a'
 
 """
-del pow_
 
 
 def fix_encoding(name, encoding=None):
